@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
-#    Monk is a lightweight schema/query framework for document databases.
-#    Copyright © 2011  Andrey Mikhaylenko
+#    Monk is an unobtrusive data modeling, manipulation and validation library.
+#    Copyright © 2011—2013  Andrey Mikhaylenko
 #
 #    This file is part of Monk.
 #
@@ -32,7 +32,18 @@ Data manipulation
     * :class:`AnyMerger`
 
 """
-import types
+from monk import compat
+
+
+__all__ = [
+    # mergers
+    'ValueMerger', 'TypeMerger', 'DictMerger', 'ListMerger', 'FuncMerger',
+    'AnyMerger',
+    # functions
+    'merge_value', 'merged',
+    # helpers
+    'unfold_list_of_dicts', 'unfold_to_list'
+]
 
 
 class ValueMerger(object):
@@ -135,8 +146,7 @@ class FuncMerger(ValueMerger):
 
     """
     def check(self):
-        func_types = types.FunctionType, types.BuiltinFunctionType
-        return isinstance(self.spec, func_types)
+        return isinstance(self.spec, compat.func_types)
 
     def process(self):
         if self.value is None:
@@ -211,7 +221,7 @@ def merged(spec, data, mergers=VALUE_MERGERS):
     if not isinstance(data, dict):
         raise TypeError('data must be a dictionary')
 
-    for key in set(spec.keys() + data.keys()):
+    for key in set(list(spec.keys()) + list(data.keys())):
         if key in spec:
             value = merge_value(spec[key], data.get(key), mergers=mergers)
         else:
@@ -222,3 +232,40 @@ def merged(spec, data, mergers=VALUE_MERGERS):
         result[key] = value
 
     return result
+
+
+def unfold_list_of_dicts(value, default_key):
+    """
+    Converts given value to a list of dictionaries as follows:
+
+    * ``[{...}]`` → ``[{...}]``
+    * ``{...}``   → ``[{...}]``
+    * ``'xyz'``   → ``[{default_key: 'xyz'}]``
+
+    """
+    if value is None:
+        return []
+    if isinstance(value, dict):
+        return [value]
+    if isinstance(value, unicode):
+        return [{default_key: value}]
+    if isinstance(value, list):
+        if not all(isinstance(x, dict) for x in value):
+            def _fix(x):
+                return {default_key: x} if isinstance(x, unicode) else x
+            return [_fix(x) for x in value]
+    return value
+
+
+def unfold_to_list(value):
+    """
+    Converts given value to a list  as follows:
+
+    * ``[x]`` → ``[x]``
+    * ``x``  → ``[x]``
+
+    """
+    if value and not isinstance(value, list):
+        return [value]
+    else:
+        return value
